/* 
   Copyright (c) 2009- Boreal - Information Strategies, published under the BSD license.
   See http://geoprisma.org/license for the full text of the license. 
*/ 
/**
 * Method: toggleLayerNode
 * Called when a layer that is member of the MapFish LayerTree has its
 * visibility changed by an other mean than the layertree itself.  Used to 
 * toogle the checkbox accordingly to the visibility of the layer.
 * 
 * The event registration MUST use  both 'layer' and 'layerTree' objects as 
 * scope parameters or else it won't work.  Ex.:
 *    myLayer.events.on({
 *        "visibilitychanged": myMapFishLayerTree.toggleLayerNode,
 *        scope: {'layer': myLayer, 'layerTree': myMapFishLayerTree}
 *    });
 *
 * Parameters:
 * this.layerTree - {<mapfish.widgets.LayerTree>}
 * this.layer     - {<OpenLayers.Layer>}
 */
mapfish.widgets.LayerTree.prototype.toggleLayerNode = function(){
    var nodeId, node, isChecked;
    
    nodeId = this.layerTree.layerToNodeIds[this.layer.name][0];
    node = this.layerTree.getNodeById(nodeId);

    if(typeof(node) != 'undefined' && typeof(node.ui) != 'undefined' && node.ui.isChecked() != this.layer.visibility){
        node.ui.toggleCheck();
    }
}

/**
 * Method: toggleSubLayerNode
 * Called when a layer that is member of the MapFish LayerTree and also has
 * sublayers has its 'params' changed other mean than the layertree itself.
 * Used to toogle the checkbox accordingly to the new 'params' of the layer.
 * 
 * Note : only WMS layers can have sublayers.
 * 
 * The event registration MUST use 'layerTree' objects as scope parameters or
 *  else it won't work.  Ex.:
 *    myLayer.map.events.on({
 *        "changelayer": myMapFishLayerTree.toggleSubLayerNode,
 *        scope: {'layerTree': myMapFishLayerTree}
 *    });
 *
 * Parameters:
 * e              - {Object} Event
 * this.layerTree - {<mapfish.widgets.LayerTree>}
 */
mapfish.widgets.LayerTree.prototype.toggleSubLayerNode = function(e){
    // if the property changed is not 'params' or the layer has no sublayers
    // in the layerTree, no need to do anything.
    if(e.property != "params" || 
       !this.layerTree.layersWithSublayers[e.layer['name']]){
        return;
    }

    var nodeId, node, isChecked, sublayers;

    sublayers = OpenLayers.Util.upperCaseObject(e.layer.params)['LAYERS'];

    for(var i=0, len = sublayers.length; i<len; i++){
        sublayer = sublayers[i];
        nodeKey = e.layer.name+":"+sublayer;

	// if sublayer doesn't have a node define in the tree, ignore it
        if (!this.layerTree.layerToNodeIds[nodeKey]) {
	    continue;
        }
        nodeId = this.layerTree.layerToNodeIds[nodeKey][0];
        node = this.layerTree.getNodeById(nodeId);

        if(!node.ui.isChecked()){
            node.ui.toggleCheck();
        }
    }
}
