<?php
/**
* Base class for fields config elements.
*
* PHP versions 5
* 
* @category  PHP
* @package   GeoPrisma  
* @author    Alexandre Dube 
* @copyright 2010, Boreal - Nippour Geomatik
* @license   http://www.geoprisma.org/license BSD License
* @link      http://www.geoprisma.org
*/

/**
* Base class for fields config elements.
* 
* @category   PHP 
* @package    GeoPrisma
* @subpackage Field 
* @author     Alexandre Dube 
*/ 
class org_geoprisma_field_Field extends org_geoprisma_ConfigElement
{
    private $m_iId = null;
    private $m_strName = null;
    private $m_strTitle = null;
    protected $m_objArrayOptions = null;

    /**
    * Create a new org_geoprisma_field_Field instance.
    * 
    * @param integer $piId      Id of the field
    * @param string  $pstrName  Name of the field
    * @param array   $pobjArray The datastore in its php array form
    * @param string  $pstrTitle (Optional) Title of the field
    * 
    * @return org_geoprisma_datastore_Datastore
    */
    public function __construct(
        $piId,
        $pstrName,
        $pobjArray,
        $pstrTitle=null
    ) {
        com_borealis_foundation_util_Assert::assertIsNotEmptyString($pstrName);

        $this->m_iId = $piId;
        $this->m_strName = $pstrName;

        if ($pstrTitle && $pstrTitle != "") 
        {
            $this->m_strTitle = $pstrTitle;
        }
        else
        {
            $this->m_strTitle = $this->m_strName;
        }

        $this->loadOptions($pobjArray); 
    }

    /**
    * From given array, create org_geoprisma_option_Option objects and append
    *     them to the options array.
    * 
    * @param array &$pobjArray Config array
    * 
    * @return void
    */
    private function loadOptions(&$pobjArray)
    {
        $this->m_objArrayOptions = new ArrayObject();

        if (isset($pobjArray['options'])) 
        {
            $objArrayOptions = $pobjArray['options'];
        }
        else
        {
            $objArrayOptions = array();
        }
        
        foreach ($objArrayOptions as $strOption => $strValue) 
        {
            $this->m_objArrayOptions->append(
                new org_geoprisma_option_Option($strOption, $strValue)
            );
        }
    }

    /**
    * Get field name
    *
    * @return string
    */
    public function getName()
    {
        return $this->m_strName;   
    }

    /**
    * Get field id
    *
    * @return string
    */
    public function getId()
    {
        return $this->m_iId;   
    }

    /**
    * Get field title
    *
    * @return string
    */
    public function getTitle()
    {
        return $this->m_strTitle;   
    }

    /**
    * Get field options
    * 
    * @return ArrayObject
    */
    public function getOptions()
    {
        return $this->m_objArrayOptions;
    }
    
    /**
    * Get a specific option using its name
    * 
    * @param string $pstrName Option name
    * 
    * @return org_geoprisma_option_Option
    */
    public function getOption($pstrName)
    {
        if ($this->m_objArrayOptions != null) 
        {
            foreach ($this->m_objArrayOptions as $objOption)   
            {
                if ($objOption->getName() == $pstrName) 
                {
                    return $objOption;
                }    
            }
        }
        throw new org_geoprisma_exception_OptionNotFoundException($pstrName);
    }

    /**
    * Add current field in XML.
    * 
    * @param DOMElement  &$pobjDomElementFields Node in which to add the field
    * @param DOMDocument &$pobjDomDocument      XML document
    * 
    * @return void
    */
    public function insertIntoXML(
        DOMElement &$pobjDomElementFields,
        DOMDocument &$pobjDomDocument
    ) {

        $objDomElementField = $pobjDomDocument->createElement('field'); 
        $pobjDomElementFields->appendChild($objDomElementField);
    
        $objDomElementName = $pobjDomDocument->createElement(
            'name', $this->getName()
        );
        $objDomElementField->appendChild($objDomElementName);

        $objDomElementTitle = $this->getDomElements(
            'title', $this->getTitle(), $pobjDomDocument
        );
        $objDomElementField->appendChild($objDomElementTitle);

        // create <options> node with its childs and append to <field>
        $objDomElementField->appendChild(
            $this->getDomElements(
                'options', $this->getOptions(), $pobjDomDocument
            )
        );
    }
}

?>
