<?php
/**
* Classe de base des service, Lier aux fichier de configuration
*
* PHP versions 5
* 
* @category  PHP
* @package   GeoPrisma  
* @author    Pascal Martin 
* @copyright 2009, Boreal - Information Strategies
* @license   http://www.geoprisma.org/license BSD License
* @link      http://www.geoprisma.org
*/

/**
* Classe de base des service, Lier aux fichier de configuration 
* 
* @category   PHP 
* @package    GeoPrisma
* @subpackage Service 
* @author     Pascal Martin 
*/ 
abstract class org_geoprisma_service_Service extends org_geoprisma_ConfigElement
{
    const BLANK_SERVICE = "__blank__";

    private $m_strName = null;
    private $m_strSource = null;
    private $m_strType = null;
    private $m_objArrayOption = null;

    /**
    * Construit un instance de org_geoprisma_service_Service
    * 
    * @param string $pstrName   Service name
    * @param string $pstrSource Source url
    * @param string $pstrType   Service type
    * @param array  $pobjArray  Config array
    * 
    * @return org_geoprisma_service_Service
    */
    public function __construct(
        $pstrName,
        $pstrSource,
        $pstrType,
        $pobjArray
    ) {
        $this->m_strName = $pstrName;
        $this->m_strSource = $pstrSource;
        $this->m_strType = $pstrType;
        $this->m_objArrayOption = new ArrayObject();

        if (isset($pobjArray['options']) && is_array($pobjArray['options'])) 
        {
            foreach ($pobjArray['options'] as $strOptionKey => $strOptionValue) 
            {
                $this->m_objArrayOption->append(
                    new org_geoprisma_option_Option(
                        $strOptionKey,
                        $strOptionValue
                    )
                );     
            }
        }
    }

    /**
    * Return the service name
    *
    * @return string
    */
    public function getName()
    {
        return $this->m_strName;   
    }
    
    /**
    * Return the service source url
    *
    * @return string
    */
    public function getSource()
    {
        return $this->m_strSource;
    }

    /**
    * Return the service type
    *
    * @return string
    */
    public function getType()
    {
        return $this->m_strType;
    }

    /**
    * Return the OpenLayers default options using this type of service.
    *
    * @param org_geoprisma_config_Config &$pobjConfig Config object
    *
    * @return array
    */
    abstract public function getDefaultLayerOptions(&$pobjConfig);

    /**
    * Ressource Options
    *
    * @return ArrayObject
    */
    public function getOptions()
    {
        return $this->m_objArrayOption;   
    }
    
    /**
    * Retourne l'option demander
    * 
    * @param string  $pstrName    Option name
    * @param boolean $pbSkipError Wheter the method should skip throwing an
    *                             error if no option is found.
    * 
    * @return org_geoprisma_option_Option
    */
    public function getOption($pstrName, $pbSkipError=false)
    {
        if ($this->getOptions() != null) 
        {
            foreach ($this->getOptions() as $objOption)   
            {
                if ($objOption->getName() == $pstrName) 
                {
                    return $objOption;
                }    
            }
        }

        if ($pbSkipError) 
        {
            return false;
        }
        else
        {
            throw new org_geoprisma_exception_OptionNotFoundException($pstrName);
        }
    }

    /**
    * Append an array of parameters to a given url.
    * 
    * @param string  $pstrURL         The url to append the parameters
    * @param boolean $pobjArrayParams Array of parameters to append to the url
    * 
    * @return string
    */
    public function urlAppend($pstrURL, $pobjArrayParams) 
    {
        $pstrURL .= (strpos($pstrURL, '?') === false) ? "?" : "&"; 
        $pstrURL .= http_build_query($pobjArrayParams);

        return $pstrURL;
    }

    /**
     * Validate that AccessFilter is true and update the source accordingly
     * There's currently 2 types of filters: ISSET and BBOX.
     * 
     * @param string  $pstrCondition  The condition string
     */
    public function validateAccessFilterCondition($pstrConditionType, $pstrInput, $pstrTrueBehavior, $pstrFalseBehavior)
    {
        $bReturn = false;
        switch($pstrConditionType)
        {
          case 'ISSET':
            $bReturn = $this->validateAccessFilterIsSet($pstrInput);
            break;
          case 'BBOX':
            $bReturn = $this->validateAccessFilterBBox($pstrInput);
            break;
          default:
            break;
        }

        if($bReturn)
        {
            $this->applyAccessFilter($pstrTrueBehavior, $pstrInput);
        }
        else
        {
            $this->applyAccessFilter($pstrFalseBehavior, $pstrInput);
        }
    }

    /**
     * validate that a value is set and not 0.
     * @param Input string
     *
     * @return bool
     */
    public function validateAccessFilterIsSet($strInput)
    {
        if($strInput)
            return true;

        return false;
    }

    /**
     * validate that a value is a BBox and inside the URL BBox.
     * @param Input string
     *
     * @return bool
     */
    public function validateAccessFilterBBox($strInput)
    {
        if(!$strInput)
            return false;

        $aszBBox = explode(',', $strInput);
        if(count($aszBBox) != 4 || 
           $aszBBox[0] > $aszBBox[2] || 
           $aszBBox[1] > $aszBBox[3])
            return false;

        // Get BBox from URL
        $strURLBBox = $_REQUEST['BBOX'];
        $aszURLBBox = explode(',', $strURLBBox);
        if($aszURLBBox[0] > $aszURLBBox[2] || $aszURLBBox[1] > $aszURLBBox[3])
            return false;

        if($aszURLBBox[0] >= $aszBBox[0] && 
           $aszURLBBox[1] >= $aszBBox[1] && 
           $aszURLBBox[2] <= $aszBBox[2] &&
           $aszURLBBox[3] <= $aszBBox[3])
            return true;

        return false;
    }

    /**
     * The Access Filter can add things in URL replace the URL with an image 
     * or return an ACL exception.
     *
     * @param $pstrBehavior What to do
     * @param $pstrInput Input string
     */
    public function applyAccessFilter($pstrBehavior, $pstrInput)
    {
        $strBehaviorType = '';
        $aszBehaviorParts = explode(':', $pstrBehavior);
        $strBehaviorType = $aszBehaviorParts[0];

        switch($strBehaviorType)
        {
          // Add this parameter to the URL with $pstrInput as value
          case 'URLPARAM':
            $this->m_strSource = $this->urlAppend($this->m_strSource, array($aszBehaviorParts[1] => $pstrInput));
            break;
          case 'IMG':
            array_shift($aszBehaviorParts);
            $this->m_strSource = implode(':', $aszBehaviorParts);
            break;
          case 'ACL':
            throw new Exception('Not Authorized by ACL');
            break;
          default:
            break;
        }
    }

    /**
    *  Ajout le service courant (this) dans le xml     
    * 
    * @param DOMElement  &$pobjDomElementServices Noeud xml ou le service doit être ajouter
    * @param DOMDocument &$pobjDomDocument        Document XMl ou le noeud xml datastore est ajouter, utiliser pour construire les noeuds
    * 
    * @return void
    */
    abstract public function insertIntoXML(DOMElement &$pobjDomElementServices, DOMDocument &$pobjDomDocument);
}

?>
